@@ -1,10 +1,10 @@
 /**
  * @file exifiron.c
  * Orientation correction and optimization of Exif JPEG images
- * @author Marko Mkel (msmakela@nic.funet.fi)
+ * @author Marko Mkel (marko.makela@iki.fi)
  */
 
-/* Copyright  2003,2004 Marko Mkel.
+/* Copyright  2003,2004,2005 Marko Mkel.
 
    This file is part of EXIFIRON, a program for orientation correction and
    shrinking of digital photographs in JPEG EXIF format.
@@ -1356,9 +1356,10 @@ exif_string_validate (JOCTET FAR* exif_data,
  * @param cinfo		decompression structure
  * @param xform		automatic (0) or enum JXFORM_CODE
  * @param timestamp	(input) pointer to timestamp to collect:
- * 0=DateTime, 1=DateTimeOriginal, 2=DateTimeDigitized
+ *			0=DateTime, 1=DateTimeOriginal, 2=DateTimeDigitized
  *			(output) the timestamp of the image in the Exif data
- * @param rmifd1	flag: 1=remove IFD1 (containing the thumbnail image)
+ * @param rmifd1	flag: 1=remove IFD1 (containing the thumbnail image),
+ *			2=do not touch Exif data, 0=normal processing
  * @param thumbname	Exif thumbnail file name (optional)
  * @return		operation to perform on the image, 0 if not Exif
  */
@@ -1530,8 +1531,7 @@ exif_parse (struct jpeg_decompress_struct* cinfo,
 	    orientation = 0;
 	  }
 	  if (exif_read2 (&exif_data[offset + 2], endian) != 3 ||
-	      exif_read4 (&exif_data[offset + 4], endian) != 1 ||
-	      exif_read2 (&exif_data[offset + 10], endian))
+	      exif_read4 (&exif_data[offset + 4], endian) != 1)
 	    fprintf (stderr, "%s: Exif: invalid Orientation record\n",
 		     srcname);
 	  else if ((orientation =
@@ -1845,12 +1845,13 @@ exif_parse (struct jpeg_decompress_struct* cinfo,
 		       srcname);
 	    else if ((thumb_size =
 		      exif_read4 (&exif_data[offset + 8], endian)) &&
-		     thumb_offset + thumb_size > exif_length + 1)
+		     thumb_offset + thumb_size > exif_length + 8)
 	      fprintf (stderr, "%s: Exif JpegIFByteCount out of bounds:"
 		       " %lu+%lu>%u\n",
 		       srcname, thumb_offset, thumb_size, exif_length);
-	    /* "exif_length + 1" tolerates an off-by-1 error
-	       of Sony DCR-PC110E */
+	    /* "exif_length + 8" tolerates an off-by-1 error
+	       of Sony DCR-PC110E and an off-by-8 error of Canon EOS 350D
+	       firmware 1.0.2, May 2005. */
 	    else {
 	      if (bytecount_ptr)
 		fprintf (stderr,
@@ -2027,28 +2028,31 @@ createfilename (const char* base,
 /** Transform the image
  * @param srcinfo	image decompression context
  * @param dstinfo	image compression context
- * @param thumbname	thumbnail file name (optional)
- * @param dstname	destination image file name (NULL = stdin)
- * @param xform		image transformation
- * @param rmifd1	flag: 1=remove Exif image file directory 1 (thumbnail)
+ * @param thumbname_	thumbnail file name (optional)
+ * @param dstname_	destination image file name (NULL = stdin/stdout)
+ * @param xform_	image transformation
+ * @param rmifd1_	flag: 1=remove Exif image file directory 1 (thumbnail),
+ *			2=do not touch Exif data, 0=normal processing
  * @param datetime	(output) timestamp to collect from Exif block
  * @param progressive	flag: use progressive coding
+ * @param optimize	flag: compute optimal Huffman coding tables
  * @return		0=ok, 1=read error, 2=write error
  */
 static int
 jpeg_transform (struct jpeg_decompress_struct* srcinfo,
 		struct jpeg_compress_struct* dstinfo,
-		const char* thumbname,
-		const char* dstname,
-		enum JXFORM_CODE xform,
-		int rmifd1,
+		const char* thumbname_,
+		const char* dstname_,
+		enum JXFORM_CODE xform_,
+		int rmifd1_,
 		time_t* datetime,
-		int progressive)
+		int progressive,
+		int optimize)
 {
   /** source file handle */
-  static FILE* src;
+  FILE* src;
   /** destination file handle */
-  static FILE* dst;
+  FILE* dst;
   /** image transformation information */
   struct jpeg_transform_info info;
   /** flag: is this a JFIF image (not Exif) */
@@ -2057,6 +2061,14 @@ jpeg_transform (struct jpeg_decompress_struct* srcinfo,
   struct jvirt_barray_control** src_coef_arrays;
   /** coefficient arrays of destination image */
   struct jvirt_barray_control** dst_coef_arrays;
+  /** thumbnail file name (optional) */
+  const char* thumbname = thumbname_;
+  /** destination image file name (NULL = stdin/stdout) */
+  const char* dstname = dstname_;
+  /** image transformation */
+  enum JXFORM_CODE xform = xform_;
+  /** flag: 1=remove Exif image file directory 1 (thumbnail) */
+  int rmifd1 = rmifd1_;
 
   /* clear possible previous errors */
   errno = 0;
@@ -2136,7 +2148,7 @@ jpeg_transform (struct jpeg_decompress_struct* srcinfo,
     }
   }
   jpeg_stdio_dest (dstinfo, dstname ? dst : stdout);
-  dstinfo->optimize_coding = TRUE;
+  dstinfo->optimize_coding = optimize;
   dstinfo->write_JFIF_header = jfif;
   if (progressive)
     jpeg_simple_progression (dstinfo);
@@ -2175,6 +2187,8 @@ main (int argc, char **argv)
   int ignopt = 0;
   /** flag: use the progressive JPEG process */
   int progressive = 1;
+  /** flag: compute optimal Huffman coding tables for the image */
+  int optimize = 1;
   /** directory for modified image files (0=replace the original files) */
   char* imagedir = 0;
   /** directory for Exif thumbnail files (0=output disabled) */
@@ -2215,7 +2229,7 @@ main (int argc, char **argv)
       case '?':
       case 'h':
 	(void) fputs
-	  ("exifiron 1.2.2\n"
+	  ("exifiron 1.2.3\n"
 	   "Orientation correction and shrinking of Exif files\n"
 	   "Usage: exifiron [options] file ...\n"
 	   "Options:\n"
@@ -2252,7 +2266,9 @@ main (int argc, char **argv)
 	   "-p"
 	   "\tprogressive JPEG (default)\n", stderr);
 	(void) fputs
-	  ("-s dir\n"
+	  ("-n"
+	   "\tdo not compute optimal Huffman coding for the image\n"
+	   "-s dir\n"
 	   "\tsave the image files in dir instead of replacing originals\n"
 	   "-S dir\n"
 	   "\tsave the Exif thumbnails in dir\n", stderr);
@@ -2279,6 +2295,8 @@ main (int argc, char **argv)
       case 'b': progressive = 0; break;
       case 'p': progressive = 1; break;
 
+      case 'n': optimize = 0; break;
+
       case 's': case 'S':
 	if (argc <= 2) {
 	  putc ('-', stderr), putc (*opts, stderr);
@@ -2373,7 +2391,7 @@ main (int argc, char **argv)
       switch (jpeg_transform (&srcinfo, &dstinfo, thumbname, dstname,
 			      transformation, rmifd1,
 			      tOutput == tdtNone ? 0 : &datetime,
-			      progressive)) {
+			      progressive, optimize)) {
       case 0:
 	break;
       case 1:
