@@ -1,9 +1,9 @@
 /** @file html.h
  * HTML output
- * @author Marko Mkel <msmakela@nic.funet.fi>
+ * @author Marko Mkel <marko.makela@iki.fi>
  */
 
-/* Copyright  2003 Marko Mkel.
+/* Copyright  2003,2005 Marko Mkel.
 
    This file is part of PHOTOMOLO, a program for generating
    thumbnail images and HTML files for browsing digital photographs.
@@ -38,10 +38,14 @@ struct html_opts
   const char* msico;
   /** character set of image comments and HTML */
   const char* charset;
+  /** generator information for HTML meta element */
+  const char* generator;
 };
 
 /** Dump the HTML code
  * @param file	the output stream
+ * @param argc	number of command-line arguments
+ * @param argv	command-line arguments
  * @param tree	the directory
  * @param level	number of '/' characters in the root directory name
  * @param info	image dimensions
@@ -56,6 +60,8 @@ struct html_opts
  */
 int
 dumphtml (FILE* file,
+	  int argc,
+	  char** argv,
 	  const struct dirtree* tree,
 	  unsigned level,
 	  const struct imageinfo* info,
