@@ -1,10 +1,10 @@
 /**
  * @file jpegnail.c
  * Remove or embed thumbnails in Exif or JFIF JPEG images
- * @author Marko Mkel (msmakela@nic.funet.fi)
+ * @author Marko Mkel <marko.makela@iki.fi>
  */
 
-/* Copyright  2003,2004 Marko Mkel.
+/* Copyright  2003,2004,2005 Marko Mkel.
 
    This file is part of JPEGNAIL, a program for removing or embedding
    JPEG thumbnails in images in Exif or JFIF format.
@@ -152,9 +152,9 @@ thumb_out_empty (struct jpeg_compress_struct* cinfo)
 /** thumbnail compression buffer */
 static JOCTET thumb_buf[16384];
 /** thumbnail length */
-static unsigned thumb_length;
+static size_t thumb_length;
 /** maximum thumbnail length */
-static unsigned thumb_maxlen;
+static size_t thumb_maxlen;
 /** maximum thumbnail image width */
 static unsigned thumb_width;
 /** maximum thumbnail image height */
@@ -402,9 +402,11 @@ exif_read4 (JOCTET FAR* data)
 
 /** Parse the Exif information
  * @param cinfo		decompression structure
+ * @param rmifd1	flag: remove IFD1 (containing the thumbnail image)
  */
 static void
-exif_parse (struct jpeg_decompress_struct* cinfo)
+exif_parse (struct jpeg_decompress_struct* cinfo,
+	    int rmifd1)
 {
   jpeg_saved_marker_ptr marker;
   exif_endian = UNKNOWN;
@@ -493,13 +495,13 @@ exif_parse (struct jpeg_decompress_struct* cinfo)
 	/* remove the existing IFD1 */
 	exif_length = ifd1_offset;
 	marker->data_length = exif_length + 6;
-	if (!thumb_width && !thumb_height)
+	if (rmifd1)
 	  exif_data[offset] =
 	    exif_data[offset + 1] =
 	    exif_data[offset + 2] =
 	    exif_data[offset + 3] = 0;
       }
-      else if (thumb_width || thumb_height) {
+      else if (!rmifd1) {
 	/* make the IFD1 pointer point to the end of the Exif block */
 	switch (exif_endian) {
 	case LSB:
@@ -558,20 +560,29 @@ createfilename (const char* base,
 /** Transform the image
  * @param srcinfo	image decompression context
  * @param dstinfo	image compression context
- * @param dstname	destination image file name (NULL = stdin)
+ * @param thumbname_	name of thumbnail file to embed
+ *			(NULL = compute thumbnail)
+ * @param dstname_	destination image file name (NULL = stdin/stdout)
  * @param progressive	flag: use progressive coding
+ * @param optimize	flag: compute optimal Huffman coding tables
  * @return		0=ok, 1=read error, 2=write error
  */
 static int
 jpeg_transform (struct jpeg_decompress_struct* srcinfo,
 		struct jpeg_compress_struct* dstinfo,
-		const char* dstname,
-		int progressive)
+		const char* thumbname_,
+		const char* dstname_,
+		int progressive,
+		int optimize)
 {
   /** input file */
   static FILE* src = 0;
   /** output file */
   static FILE* dst = 0;
+  /** name of thumbnail file to embed (NULL = compute thumbnail) */
+  const char* thumbname = thumbname_;
+  /** destination image file name (NULL = stdin/stdout) */
+  const char* dstname = dstname_;
 
   /* clear possible previous errors */
   errno = 0;
@@ -616,9 +627,33 @@ jpeg_transform (struct jpeg_decompress_struct* srcinfo,
       jpeg_save_markers (srcinfo, m, 0xFFFF);
   }
   (void) jpeg_read_header (srcinfo, TRUE);
-  exif_parse (srcinfo);
-  /* create thumbnail image for JFIF or Exif files */
-  if ((thumb_width || thumb_height) &&
+  exif_parse (srcinfo, !thumbname && !thumb_width && !thumb_height);
+  /* read or create thumbnail image for JFIF or Exif files */
+  if (thumbname) {
+    FILE* thumbfile;
+    if (!(thumbfile = fopen (thumbname, "rb"))) {
+      (void) fputs (thumbname, stderr), (void) fflush (stderr);
+      perror (": open (reading)");
+      goto ignore;
+    }
+    thumb_length = fread (thumb_buf, 1, sizeof thumb_buf, thumbfile);
+    if (ferror (thumbfile)) {
+      (void) fputs (thumbname, stderr), (void) fflush (stderr);
+      perror (": read");
+      (void) fclose (thumbfile);
+      goto ignore;
+    }
+
+    (void) fclose (thumbfile);
+
+    if (thumb_length == sizeof thumb_buf) {
+      fprintf (stderr, "%s: thumbnail image is too large\n", thumbname);
+      goto ignore;
+    }
+    else if (!thumb_length)
+      fprintf (stderr, "%s: ignoring empty thumbnail image file\n", thumbname);
+  }
+  else if ((thumb_width || thumb_height) &&
       (exif_endian != UNKNOWN || srcinfo->saw_JFIF_marker)) {
     thumb_create (srcinfo);
     if (!thumb_length)
@@ -632,7 +667,7 @@ jpeg_transform (struct jpeg_decompress_struct* srcinfo,
     (void) jpeg_finish_decompress (srcinfo);
     jpeg_stdio_src (srcinfo, src);
     (void) jpeg_read_header (srcinfo, TRUE);
-    exif_parse (srcinfo); /* need to patch the Exif marker again */
+    exif_parse (srcinfo, 0); /* need to patch the Exif marker again */
   }
   else
     thumb_length = 0;
@@ -657,7 +692,7 @@ jpeg_transform (struct jpeg_decompress_struct* srcinfo,
   }
   jpeg_stdio_dest (dstinfo, dstname ? dst : stdout);
   jpeg_copy_critical_parameters (srcinfo, dstinfo);
-  dstinfo->optimize_coding = TRUE;
+  dstinfo->optimize_coding = optimize;
   dstinfo->write_JFIF_header =
     srcinfo->saw_JFIF_marker && exif_endian == UNKNOWN;
   dstinfo->JFIF_major_version = (UINT8) 1;
@@ -710,10 +745,20 @@ main (int argc, char **argv)
   int ignopt = 0;
   /** flag: use the progressive JPEG process */
   int progressive = 1;
+  /** flag: compute optimal Huffman coding tables for the image */
+  int optimize = 1;
   /** directory for modified image files (0=replace the original files) */
   char* imagedir = 0;
   /** imagedir length */
   unsigned imagedirlen = 0;
+  /** directory for thumbnail image files (0=compute thumbnails) */
+  char* thumbdir = 0;
+  /** thumbdir length */
+  unsigned thumbdirlen = 0;
+  /** destination file name */
+  char* dstname = 0;
+  /** thumbnail file name */
+  char* thumbname = 0;
 
   srcinfo.err = jpeg_std_error (&jsrcerr.pub);
   jsrcerr.pub.addon_message_table = exif_message_table;
@@ -749,7 +794,7 @@ main (int argc, char **argv)
       case '?':
       case 'h':
 	(void) fputs
-	  ("jpegnail 1.2.2\n"
+	  ("jpegnail 1.2.3\n"
 	   "Remove or embed thumbnails in Exif or JFIF JPEG images\n"
 	   "Usage: jpegnail [options] file ...\n"
 	   "Options:\n"
@@ -765,6 +810,10 @@ main (int argc, char **argv)
 	   "\tbaseline JPEG\n"
 	   "-p"
 	   "\tprogressive JPEG (default)\n"
+	   "-n"
+	   "\tdo not compute optimal Huffman coding for the image\n"
+	   "-S dir\n"
+	   "\tread image thumbnails from dir instead of computing them\n"
 	   "-s dir\n"
 	   "\tsave the image files in dir instead of replacing originals\n",
 	   stderr);
@@ -773,7 +822,10 @@ main (int argc, char **argv)
       case 'b': progressive = 0; break;
       case 'p': progressive = 1; break;
 
+      case 'n': optimize = 0; break;
+
       case 's':
+      case 'S':
 	if (argc <= 2) {
 	missing:
 	  (void) putc ('-', stderr), (void) putc (*opts, stderr);
@@ -784,16 +836,23 @@ main (int argc, char **argv)
 	argc--;
 	if (!**++argv) {
 	  /* empty string argument => restore default behaviour */
-	  imagedir = 0, imagedirlen = 0;
+	  if (*opts == 's')
+	    imagedir = 0, imagedirlen = 0;
+	  else
+	    thumbdir = 0, thumbdirlen = 0;
 	}
 	else {
 	  /* make sure that the specified directory exists */
+	  char* dir;
+	  unsigned dirlen;
 #if defined WIN32 || defined __WIN32
 	  char* s = *argv;
 	  while (*s) if (*s++ == '\\') s[-1] = '/';
 #endif
-	  imagedirlen = strlen (imagedir = *argv);
-	  if (mkdir (*argv
+	  dirlen = strlen (dir = *argv);
+	  /* try to create directory for output images, but not for inputs */
+	  if (*opts == 's' &&
+	      mkdir (*argv
 #if !defined WIN32 && !defined __WIN32
 		     , 0777
 #endif
@@ -804,14 +863,14 @@ main (int argc, char **argv)
 	  }
 	  else {
 	    struct stat statbuf;
-	    char* s = malloc (imagedirlen + 3);
+	    char* s = malloc (dirlen + 3);
 	    if (!s) {
 	      (void) putc ('-', stderr), (void) putc (*opts, stderr);
 	      (void) fputs (": out of memory\n", stderr);
 	      goto quit;
 	    }
-	    memcpy (s, *argv, imagedirlen);
-	    memcpy (s + imagedirlen, "/.", 3);
+	    memcpy (s, *argv, dirlen);
+	    memcpy (s + dirlen, "/.", 3);
 	    if (stat (s, &statbuf)) {
 	      (void) fputs (s, stderr), (void) fflush (stderr);
 	      perror (": stat");
@@ -820,6 +879,11 @@ main (int argc, char **argv)
 	    }
 	    free (s);
 	  }
+
+	  if (*opts == 's')
+	    imagedir = dir, imagedirlen = dirlen;
+	  else
+	    thumbdir = dir, thumbdirlen = dirlen;
 	}
 	break;
       case 'q':
@@ -857,34 +921,58 @@ main (int argc, char **argv)
 
     /* if opts != 0 at this point, it denotes the file name '-' */
     srcname = *argv;
-    {
-      /** destination file name */
-      char* dstname = 0;
-      if (!opts) { /* file name not '-' */
-	unsigned len = strlen (*argv);
-	if (!(dstname = imagedir
-	      ? createfilename (*argv, len, imagedir, imagedirlen)
-	      : malloc (len + 1))) {
-	  (void) fputs (*argv, stderr);
-	  (void) fputs (": out of memory\n", stderr);
-	  goto quit;
-	}
-	if (!imagedir) {
-	  memcpy (dstname, *argv, len + 1);
-	  dstname[len - 1] = '0'; /* change the last character to '0' */
+    if (!opts) { /* file name not '-' */
+      unsigned len = strlen (*argv);
+      if (!(dstname = imagedir
+	    ? createfilename (*argv, len, imagedir, imagedirlen)
+	    : malloc (len + 1))) {
+      oom:
+	(void) fputs (*argv, stderr);
+	(void) fputs (": out of memory\n", stderr);
+	goto quit;
+      }
+      if (!imagedir) {
+	memcpy (dstname, *argv, len + 1);
+	dstname[len - 1] = '0'; /* change the last character to '0' */
+      }
+      if (thumbdir) {
+	/* read thumbnail images from a directory */
+	if (!(thumbname =
+	      createfilename (*argv, len, thumbdir, thumbdirlen))) {
+	  free (dstname);
+	  goto oom;
 	}
       }
-      jpeg_transform (&srcinfo, &dstinfo, dstname, progressive);
-      if (!opts) {
-	(void) touch_file (dstname, srcname);
-	if (!imagedir && rename_file (dstname, srcname)) {
-	quit:
-	  free (dstname), dstname = 0;
-	  status = 3;
-	  goto done;
-	}
+      else
+	thumbname = 0;
+    }
+    else
+      dstname = thumbname = 0;
+
+    switch (jpeg_transform (&srcinfo, &dstinfo, thumbname, dstname,
+			    progressive, optimize)) {
+    case 0:
+      free (thumbname);
+      break;
+    case 1:
+      free (thumbname);
+      free (dstname);
+      continue;
+    default:
+      free (thumbname);
+      free (dstname);
+      status = 2;
+      goto done;
+    }
+    if (!opts) {
+      (void) touch_file (dstname, srcname);
+      if (!imagedir && rename_file (dstname, srcname)) {
+      quit:
 	free (dstname);
+	status = 3;
+	goto done;
       }
+      free (dstname);
     }
     (void) jpeg_finish_decompress (&srcinfo);
   }
