@@ -1,10 +1,10 @@
 /**
  * @file photomolo.c
  * The main program
- * @author Marko Mkel <msmakela@nic.funet.fi>
+ * @author Marko Mkel <marko.makela@iki.fi>
  */
 
-/* Copyright  2003,2004 Marko Mkel.
+/* Copyright  2003,2004,2005 Marko Mkel.
 
    This file is part of PHOTOMOLO, a program for generating
    thumbnail images and HTML files for browsing digital photographs.
@@ -38,6 +38,8 @@
 #include "spec.h"
 #include "html.h"
 
+#define PHOTOMOLO_VERSION "pHoToMoLo 1.2.3"
+
 /** The libjpeg decompression context */
 static struct jpeg_decompress_struct cin;
 /** The libjpeg compression context */
@@ -55,7 +57,10 @@ static struct html_opts hopts = {
   "index.css",
   0, /* "icon" file name */
   0, /* "shortcut icon" file name */
-  "iso-8859-1" /* character set */
+  "iso-8859-1", /* character set */
+  PHOTOMOLO_VERSION ";\n "
+  "http://www.funet.fi/pub/sci/graphics/packages/photomolo/photomolo.html;\n"
+  /* contents of the HTML META Generator element */
 };
 
 /** Length of hopts.html, plus 1 */
@@ -68,6 +73,11 @@ static int dump_html = 1;
 /** flag: remove thumbnail directories */
 static int remove_thumbdirs = 0;
 
+/** number of command-line arguments */
+static int main_argc;
+/** command-line arguments */
+static char** main_argv;
+
 /** Cleanup function */
 static void
 cleanup (void)
@@ -236,7 +246,8 @@ dumphtmls (const struct dirtree* tree,
       free (filename);
       return 1;
     }
-    status = dumphtml (f, tree, level, info, comm, &hopts, thumbs, spec,
+    status = dumphtml (f, main_argc, main_argv,
+		       tree, level, info, comm, &hopts, thumbs, spec,
 		       top, prev, next);
     if (fclose (f) || status) {
       (void) fputs (filename, stderr), (void) fflush (stderr);
@@ -399,7 +410,7 @@ create_css (const char* path)
   static const char css1[] =
     "/*\n"
     " * Sample style sheet for pHoToMoLo-generated image gallery HTML\n"
-    " * Written in 2003 by Marko Mkel (msmakela@nic.funet.fi).\n"
+    " * Written in 2003 by Marko Mkel (marko.makela@iki.fi).\n"
     " * This file is in the public domain.\n"
     " * The layout has been tested with the following browsers:\n"
     " * Lynx, Netscape 4, Opera 6, IE 5.01, IE 5.5, IE 6.0, Mozilla 1.x.\n"
@@ -477,7 +488,7 @@ main (int argc, char** argv)
   if (argc == 1) {
   Usage:
     (void) fputs
-      ("pHoToMoLo 1.2.2\n"
+      (PHOTOMOLO_VERSION "\n"
        "Create thumbnails and HTML navigation\n"
        "Usage: photomolo [options] directory1 directory2 ...\n"
        "Options:\n"
@@ -509,6 +520,9 @@ main (int argc, char** argv)
     return 0;
   }
 
+  main_argc = argc;
+  main_argv = argv;
+
   /* process the options */
   for (argv++; argc > 1 && **argv == '-'; argv++, argc--) {
     const char* opts = *argv;
